from pupa.models import Person
from .base import BaseImporter, db


class PersonImporter(BaseImporter):
    _type = 'person'
    _model_class = Person

    def __init__(self, jurisdiction_id):
        super(PersonImporter, self).__init__(jurisdiction_id)
        # get list of all people w/ memberships in this org
        self.person_ids = db.memberships.find(
            {'jurisdiction_id': jurisdiction_id}).distinct('person_id')

    def prepare_object_from_json(self, obj):
        if 'party' in obj:
            obj.pop('party')
        return obj

    def get_db_spec(self, person):
        spec = {'$or': [{'name': person.name},
                        {'other_names': person.name}],
                '_id': {'$in': self.person_ids}}

        if hasattr(person, 'chamber') and person.chamber is not None:
            spec['chamber'] = person.chamber

        if hasattr(person, 'post_id') and person.post_id is not None:
            spec['post_id'] = person.post_id

        return spec



"""
    Schema for vote objects.
"""

from .common import sources


VOTE_TYPES = ['passage', 'amendment', 'reading:2', 'reading:3',
              'veto_override', 'other']
ROLLCALL_TYPES = ['yes', 'no', 'abstain', 'not-voting', 'other']


schema = {
    "description": "schema for vote data",
    "type": "object",
    "_order": (
        ('Basic Fields', ["organization", "organization_id", "_type", "session",
                          "chamber", "date", "motion", "type", "passed"]),
        ('Common Fields', ['updated_at', 'created_at', 'sources']),
        ('Relationship to Bill', ["bill"]),
        ('Vote Counts', ["vote_counts", "roll_call"])
    ),
    "properties": {

        'organization': {"type": ["string", "null"],
                      "description": "name of the voting organization"},

        'organization_id': {"type": ["string", "null"],
                         "description": "id of the voting organization"},

        '_type': {"enum": ["vote"], "type": "string",
               "description": "All vote objects must have a _type field set to vote." },

        'session': {"type": "string",
                 "description": "Associated with one of the jurisdiction's sessions"},

        'updated_at': {"type": ["string", "datetime"], "required": False,
                    "description": "the time that the object was last updated",
                   },

        'created_at': {"type": ["string", "datetime"], "required": False,
                    "description": "the time that this object was first created" },

        'chamber': {
            "enum": ["upper", "lower", "joint"], "type": ["string", "null"],
            "description": "chamber vote took place in (if legislature is bicameral, otherwise null)",
        },

        'date': {"pattern": "^[0-9]{4}(-[0-9]{2}){0,2}$", "type": "string",
              "description": "date of the action" },

        'motion': {"type": "string", "description": "description of motion (from upstream source)"},

        'type': {"items": {"type": "string", "enum": VOTE_TYPES}, "type": "array",
                 "description": "array of types" },

        'passed': {"type": "boolean", "description": "boolean indicating if vote passed"},

        'bill': {
            "type": ["object", "null"],
            "properties": {
                "id": {"type": ["string", "null"],
                       "description": ("bill's internal id if bill was matched with an object in "
                                       "the database")

                      },
                "name": {"type": "string", "description": "bill name (e.g. HB 21)"},
                "chamber": {
                    "enum": ["upper", "lower"], "type": ["string", "null"],
                    "description": "bill's chamber if vote was on a bill (and legislature is bicameral, otherwise null)"
                },
            },
            "description": ("Related bill, votes will have a non-null bill object if"
                            "they are related to a bill. Bills will have the following fields:"),
        },

        'vote_counts': {
            "items": {
                "properties": {
                    "vote_type": {"type": "string", "enum": ROLLCALL_TYPES,
                                  "description": "(e.g. yes, no, not-voting)"
                                 },
                    "count": {"type": "integer", "minimum": 0,
                              "description": "number of people voting this way",
                             }
                },
                "type": "object"
            },
            "description": ("list of objects with vote_type and count properties"),
        },

        'roll_call': {
            "items": {
                "type": "object",
                "properties": {
                    "vote_type": {"type": "string", "enum": ROLLCALL_TYPES,
                                  "description": "(e.g. yes, no, not-voting)"
                                 },

                    #     * **name** - person's name as provided by source
                    #     * **id** - person's internal id if they've been
                    #       matched to an entity in the database
                    "person": {
                        "type": "object",
                        "properties": {
                            "name": {"type": "string", "description": "person's name as provided by the source"},
                            "id": {"type": ["string", "null"], "description": "person's internal id if they've been matched to an entity in the database"},
                        },
                        "description": "person object representing the voter",
                    }
                },
            },
            "description": "list of individual legislator votes",
        },

        'sources': sources,
    }
}



from .base import BaseModel
from .schemas.organization import schema


class Organization(BaseModel):
    """
    A single popolo encoded Organization
    """

    __slots__ = ('classification', 'dissolution_date', 'founding_date',
                 'identifiers', 'name', 'other_names', 'parent_id', 'chamber',
                 'posts', '_openstates_id', 'contact_details', 'division_id',
                 'abbreviation', 'jurisdiction_id', 'identifiers', 'links',
                 'image',
                )

    _post_slots = ('end_date', 'id', 'label', 'role',
                   'start_date', 'chamber', 'division_id',
                   'num_seats')

    _type = 'organization'
    _schema = schema
    _collection = 'organizations'

    def __init__(self, name, **kwargs):
        """
        Constructor for the Organization object.
        """
        super(Organization, self).__init__()
        self.links = []
        self.name = name
        self.classification = None
        self.founding_date = None
        self.dissolution_date = None
        self.parent_id = None
        self.image = None
        self.other_names = []
        self.identifiers = []
        self.posts = []
        self.contact_details = []
        self._related = []
        for k, v in kwargs.items():
            setattr(self, k, v)

    def __str__(self):
        return self.name
    __unicode__ = __str__

    def __repr__(self):
        as_dict = self.as_dict()
        list(map(as_dict.pop, ('_type', '_id', 'name')))
        args = (self.__class__.__name__, self.name, as_dict)
        return '%s(name=%r, **%r)' % args

    def validate(self):
        schema = None
        if self.classification in ['party']:
            # Parties are funny objects. They don't need sources, since
            # they're really not something we scrape directly (well, ever).
            # As a result, we're not going to enforce placing sources on a
            # party object.
            schema = self._schema.copy()
            schema['properties'] = schema['properties'].copy()
            schema['properties'].pop('sources')
        return super(Organization, self).validate(schema=schema)

    @property
    def parent(self):
        return self.parent_id

    @parent.setter
    def parent(self, val):
        self.parent_id = val._id

    def add_link(self, url, note=None):
        self.links.append({"note": note, "url": url})

    def add_identifier(self, identifier, scheme=None):
        data = {"identifier": identifier}
        if scheme:
            data['scheme'] = scheme
        self.identifiers.append(data)

    def add_post(self, label, role, **kwargs):
        post = {"label": label, "role": role}
        for k, v in kwargs.items():
            if k not in self._post_slots:
                raise AttributeError(
                    '{0} not a valid kwarg for add_post'.format(k))
            post[k] = v
        self.posts.append(post)

    def add_contact_detail(self, type, value, note):
        self.contact_details.append({"type": type,
                                     "value": value,
                                     "note": note})



import os
import sys
import logging
import logging.config

import pymongo

from . import default_settings


class Settings(object):
    def __init__(self):
        pass

    def __setattr__(self, attr, val):
        super(Settings, self).__setattr__(attr, val)
        # if logging config is changed, reconfigure logging
        if attr == 'LOGGING_CONFIG':
            logging.config.dictConfig(self.LOGGING_CONFIG)

    def update(self, module):
        if isinstance(module, dict):
            for setting, val in module.items():
                if setting.isupper() and val is not None:
                    setattr(self, setting, val)
        else:
            for setting in dir(module):
                if setting.isupper():
                    val = getattr(module, setting)
                    if val is not None:
                        setattr(self, setting, val)

settings = Settings()
settings.update(default_settings)

try:
    sys.path.insert(0, os.getcwd())
    import pupa_settings
    settings.update(pupa_settings)
    sys.path.pop(0)
except ImportError:
    logging.warning('no pupa_settings file found, continuing with defaults..')

db = None
elasticsearch = None


class ErrorProxy(object):
    def __init__(self, error):
        self.error = error

    def __getattr__(self, attr):
        raise self.error


def _configure_db(host, port, db_name):
    global db

    try:
        conn = pymongo.Connection(host, port)
        db = conn[db_name]
    # return a dummy NoDB object if we couldn't connect
    except (pymongo.errors.AutoReconnect,
            pymongo.errors.ConnectionFailure) as e:
        db = ErrorProxy(e)


def _configure_es(host, timeout):
    import pyelasticsearch
    global elasticsearch
    try:
        elasticsearch = pyelasticsearch.ElasticSearch(host, timeout=timeout,
                                                      revival_delay=0)
    except Exception as e:
        elasticsearch = ErrorProxy(e)


_configure_db(settings.MONGO_HOST, settings.MONGO_PORT,
              settings.MONGO_DATABASE)
if settings.ENABLE_ELASTICSEARCH:
    _configure_es(settings.ELASTICSEARCH_HOST, settings.ELASTICSEARCH_TIMEOUT)
